using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

namespace Snivilisation {
	enum GameState { NORMAL, SHOW_POPULATION, SHOW_TAX, SHOW_SCIENCE };

	abstract class SnivObject {
		public int XPos;
		public int YPos;

		public abstract void Update();
		public abstract void Draw();
	}

	class SnivCity : SnivObject {
		public string _Name;

		public string Name {
			get {
				return _Name;
			}

			set {
				_Name = value;
			}
		}

		private float Size = 100;

		public override void Draw() {
			if (Size < 1000) {
				Console.ForegroundColor = ConsoleColor.DarkRed;
				Console.Write(".");
			} else if (Size < 10000) {
				Console.ForegroundColor = ConsoleColor.Red;
				Console.Write(":");
			} else if (Size < 100000) {
				Console.ForegroundColor = ConsoleColor.DarkYellow;
				Console.Write("%");
			} else if (Size < 1000000) {
				Console.ForegroundColor = ConsoleColor.Yellow;
				Console.Write("*");
			} else if (Size < 10000000) {
				Console.ForegroundColor = ConsoleColor.DarkGreen;
				Console.Write("@");
			} else {
				Console.ForegroundColor = ConsoleColor.Green;
				Console.Write("#");
			}
		}

		public override void Update() {
			float growthrate = 25;
			growthrate -= Snivilization.TaxRate;

			Size *= 1 + (growthrate / 200.0f);
		}

		public int GetTax() {
			return (int)Math.Round((Size / 400.0f) * Snivilization.TaxRate);
		}

		public int GetScience() {
			return (int)Math.Round((GetTax() / 100.0f) * Snivilization.ScienceRate);
		}

		public int GetSize() {
			return (int)Math.Floor(Size);
		}
	}

	abstract class SnivUnit : SnivObject {
		public int Type;

		public abstract bool CanMove();
		public bool Move() {
			// move the object here
			return true;
		}
	}

	class Snivilization {
		public int GameWidth = 100;
		public int GameHeight = 40;

		public static Random Rand = new Random();

		public static int TaxRate = 8;
		public static int ScienceRate = 25;

		public int TotalCash;
		public int CashLastTurn;
		public int ScienceLastTurn;

		public GameState State = GameState.NORMAL;

		public List<String> CityNames1 = new List<string>();
		public List<String> CityNames2 = new List<string>();
		public List<String> CityNames3 = new List<string>();

		public List<SnivCity> Cities = new List<SnivCity>();
		public List<Point> CityLocations = new List<Point>();

		static void Main(string[] args) {
			Snivilization game = new Snivilization();
			game.Run();
		}

		void Run() {
			// generate city locations, to avoid placing cities twice
			for (int i = 0; i < GameHeight; ++i) {
				for (int j = 0; j < GameWidth; ++j) {
					CityLocations.Add(new Point(j, i));
				}
			}

			// generate city names
			CityNames1.Add("Lon");
			CityNames1.Add("Bir");
			CityNames1.Add("Edin");
			CityNames1.Add("Glas");
			CityNames1.Add("Car");
			CityNames1.Add("Bel");
			CityNames1.Add("Man");

			CityNames2.Add("ming");
			CityNames2.Add("don");
			CityNames2.Add("cast");
			CityNames2.Add("chest");
			CityNames2.Add("fast");

			CityNames3.Add("ol");
			CityNames3.Add("er");
			CityNames3.Add("on");

			DrawScreen();

			while (true) {
				ConsoleKeyInfo key = System.Console.ReadKey(true);

				switch (key.Key) {
					case ConsoleKey.N:
						CreateCity();
						DrawScreen();
						break;

					case ConsoleKey.P:
						State = GameState.SHOW_POPULATION;
						DrawScreen();
						break;

					case ConsoleKey.S:
						State = GameState.SHOW_SCIENCE;
						DrawScreen();
						break;

					case ConsoleKey.T:
						State = GameState.SHOW_TAX;
						DrawScreen();
						break;

					case ConsoleKey.UpArrow:
						++TaxRate;
						break;

					case ConsoleKey.DownArrow:
						--TaxRate;
						break;

					case ConsoleKey.C:
						State = GameState.NORMAL;
						UpdateCities();
						DrawScreen();
						break;
				}
			}
		}

		public void DrawScreen() {
			Console.Clear();

			SnivCity foundcity;

			for (int i = 0; i < GameHeight; ++i) {
				for (int j = 0; j < GameWidth; ++j) {
					foundcity = null;

					foreach(SnivCity city in Cities) {
						if (city.YPos == i && city.XPos == j) {
							foundcity = city;
							break;
						}				
					}

					if (foundcity != null) {
						foundcity.Draw();
					} else {
						Console.Write(" ");
					}
				}

				Console.Write("\n");
			}

			Console.WriteLine("");

			switch (State) {
				case GameState.NORMAL:
					Console.ForegroundColor = ConsoleColor.Cyan;
					Console.WriteLine("Income last turn: " + CashLastTurn);
					Console.WriteLine("Science last turn: " + ScienceLastTurn);
					Console.WriteLine();
					Console.ForegroundColor = ConsoleColor.White;
					Console.WriteLine("Total cash: " + TotalCash);
					break;

				case GameState.SHOW_POPULATION:
					Console.ForegroundColor = ConsoleColor.Cyan;
					Console.WriteLine("Population status");
					Console.ForegroundColor = ConsoleColor.White;

					foreach(SnivCity city in Cities) {
						Console.WriteLine(city.Name + ": " + city.GetSize());
					}

					break;

				case GameState.SHOW_SCIENCE:
					Console.ForegroundColor = ConsoleColor.Cyan;
					Console.WriteLine("Science status");
					Console.ForegroundColor = ConsoleColor.White;

					foreach(SnivCity city in Cities) {
						Console.WriteLine(city.Name + ": " + city.GetScience());
					}

					break;

				case GameState.SHOW_TAX:
					Console.ForegroundColor = ConsoleColor.Cyan;
					Console.WriteLine("Tax status");
					Console.ForegroundColor = ConsoleColor.White;

					foreach(SnivCity city in Cities) {
						Console.WriteLine(city.Name + ": " + city.GetTax());
					}

					break;
			}
		}

		public void CreateCity() {
			// if there are no more city positions available, bail out
			if (CityLocations.Count == 0) return;

			int newpos = Rand.Next(0, CityLocations.Count);
			Point citypos = CityLocations[newpos];
			CityLocations.RemoveAt(newpos);

			
			SnivCity city = new SnivCity();
			city.XPos = citypos.X;
			city.YPos = citypos.Y;

			city.Name = CityNames1[Rand.Next(0, CityNames1.Count)];
			city.Name += CityNames2[Rand.Next(0, CityNames2.Count)];
			if (Rand.Next(0, 3) == 0) city.Name += CityNames3[Rand.Next(0, CityNames3.Count)];

			Cities.Add(city);
		}

		public void UpdateCities() {
			CashLastTurn = 0;
			ScienceLastTurn = 0;

			foreach (SnivCity city in Cities) {
				city.Update();

				CashLastTurn += city.GetTax();
				ScienceLastTurn += city.GetScience();
			}

			TotalCash += CashLastTurn;
		}
	}
}
